@@ -1,7 +1,7 @@
 
 # The R Commander and command logger
 
-# last modified 2014-10-07 by John Fox
+# last modified 2015-10-18 by John Fox
 
 # contributions by Milan Bouchet-Valat, Richard Heiberger, Duncan Murdoch, Erich Neuwirth, Brian Ripley
 
@@ -40,6 +40,10 @@ Commander <- function(){
             !is.null(get("commanderWindow", RcmdrEnv()))) {
         return(invisible(NULL))
     }
+    
+    # check for auxiliary software
+    putRcmdr("capabilities", RcmdrCapabilities())
+    
     # the following function used to apply Rcmdr options with specified defaults
     #   if global == TRUE, store option
     setOption <- function(option, default, global=TRUE) {
@@ -282,16 +286,16 @@ Commander <- function(){
         tkfont.configure("RcmdrTitleFont", weight="bold")
     }
     else tkfont.configure("RcmdrTitleFont", weight="normal")
-#     .Tcl("ttk::style configure TLabelFrame -font RcmdrTitleFont")
-#     .Tcl(paste("ttk::style configure TLabelframe -foreground", title.color))
     .Tcl("ttk::style configure TNotebook.Tab -font RcmdrDefaultFont")
     .Tcl(paste("ttk::style configure TNotebook.Tab -foreground", title.color))
     setOption("number.messages", TRUE)
     setOption("log.commands", TRUE)
     setOption("use.knitr", FALSE)
     setOption("use.markdown", !getRcmdr("use.knitr"))
-#    if ((!packageAvailable("markdown")) || (!packageAvailable("knitr"))) putRcmdr("use.markdown", FALSE)
-#    if (!packageAvailable("knitr")) putRcmdr("use.knitr", FALSE)
+    if ((!packageAvailable("markdown") && !packageAvailable("rmarkdown")) || (!packageAvailable("knitr"))) 
+        putRcmdr("use.markdown", FALSE)
+    if (!packageAvailable("knitr") || !getRcmdr("capabilities")$pdflatex) putRcmdr("use.knitr", FALSE)
+    setOption("rmd.output.format", "html")
     putRcmdr("startNewCommandBlock", TRUE)
     putRcmdr("startNewKnitrCommandBlock", TRUE)
     putRcmdr("rmd.generated", FALSE)
@@ -370,6 +374,25 @@ Commander <- function(){
     }
     setOption("tkwait.dialog", FALSE)
     if (getRcmdr("tkwait.dialog")) putRcmdr("editDataset.threshold", 0)
+    if (MacOSXP()){
+        #       PATH <- system2("/usr/libexec/path_helper", "-s", stdout=TRUE)
+        #       PATH <- sub("\"; export PATH;$", "", sub("^PATH=\\\"", "", PATH))
+        #       Sys.setenv(PATH=PATH)
+        PATH <- Sys.getenv("PATH")
+        PATH <- unlist(strsplit(PATH, .Platform$path.sep, fixed=TRUE))
+        if (MacOSXP("15.0.0")){
+            if (length(grep("^/Library/TeX/texbin$", PATH)) == 0) {
+                PATH[length(PATH) + 1] <- "/Library/TeX/texbin"
+                Sys.setenv(PATH=paste(PATH, collapse=.Platform$path.sep))
+            }
+        }
+        else{
+            if (length(grep("^/usr/texbin$", PATH)) == 0) {
+                PATH[length(PATH) + 1] <- "/usr/texbin"
+                Sys.setenv(PATH=paste(PATH, collapse=.Platform$path.sep))
+            }
+        }
+    }
     
     # source additional .R files, plug-ins preferred
     source.files <- list.files(etc, pattern="\\.[Rr]$")
@@ -630,7 +653,7 @@ Commander <- function(){
     contextMenuRmd <- function(){
         .rmd <- RmdWindow()
         contextMenu <- tkmenu(tkmenu(.rmd), tearoff=FALSE)
-        tkadd(contextMenu, "command", label=gettextRcmdr("Generate HTML report"), command=onSubmit)
+        tkadd(contextMenu, "command", label=gettextRcmdr("Generate report"), command=onSubmit)
         tkadd(contextMenu, "command", label=gettextRcmdr("Edit R Markdown document"), command=editMarkdown)
         tkadd(contextMenu, "command", label=gettextRcmdr("Remove last Markdown command block"), command=removeLastRmdBlock)
         tkadd(contextMenu, "separator")
@@ -785,7 +808,8 @@ Commander <- function(){
         font=getRcmdr("logFont"), height=log.height, width=log.width, wrap="none", undo=TRUE))
     .rmd <- RmdWindow()
     rmd.template <- setOption("rmd.template", 
-        system.file("etc", "Rcmdr-Markdown-Template.Rmd", package="Rcmdr"))
+        system.file("etc", if (getRcmdr("capabilities")$pandoc) "Rcmdr-RMarkdown-Template.Rmd"
+            else "Rcmdr-Markdown-Template.Rmd", package="Rcmdr"))
     template <- paste(readLines(rmd.template), collapse="\n")
     tkinsert(.rmd, "end", template)
     putRcmdr("markdown.output", FALSE)
@@ -822,7 +846,7 @@ Commander <- function(){
     
     tkbind(CommanderWindow(), "<Button-1>", function() {
         if (as.character(tkselect(notebook)) == logFrame$ID) tclvalue(submitButtonLabel) <- gettextRcmdr("Submit")
-        if (as.character(tkselect(notebook)) == RmdFrame$ID) tclvalue(submitButtonLabel) <- gettextRcmdr("Generate HTML report")
+        if (as.character(tkselect(notebook)) == RmdFrame$ID) tclvalue(submitButtonLabel) <- gettextRcmdr("Generate report")
         if (as.character(tkselect(notebook)) == RnwFrame$ID) tclvalue(submitButtonLabel) <- gettextRcmdr("Generate PDF report")
     })
     putRcmdr("outputWindow", tktext(outputFrame, bg="white", foreground=getRcmdr("output.text.color"),
@@ -1006,7 +1030,7 @@ Commander <- function(){
         putRcmdr(".commander.done", tclVar("0"))
         tkwait.variable(getRcmdr(".commander.done"))
     }
-    Message(paste(gettextRcmdr("R Commander Version "), getRcmdr("RcmdrVersion"), ": ", date(), sep=""))
+    Message(paste(gettextRcmdr("R Commander Version "), " ", getRcmdr("RcmdrVersion"), ": ", date(), sep=""))
     if (.Platform$GUI == "Rgui"  && ismdi()) Message(gettextRcmdr(
         "The Windows version of the R Commander works best under RGui\nwith the single-document interface (SDI); see ?Commander."),
         type="warning")
@@ -1325,10 +1349,10 @@ pushOutput <- function(element) {
     putRcmdr("outputStack", stack)
 }
 
-popOutput <- function(){
+popOutput <- function(keep=FALSE){
     stack <- getRcmdr("outputStack")
     lastOutput <- stack[[1]]
-    putRcmdr("outputStack", c(stack[-1], NA))
+    if (!keep) putRcmdr("outputStack", c(stack[-1], NA))
     lastOutput
 }
 
@@ -1338,9 +1362,9 @@ pushCommand <- function(element) {
     putRcmdr("commandStack", stack)
 }
 
-popCommand <- function(){
+popCommand <- function(keep=FALSE){
     stack <- getRcmdr("commandStack")
     lastCommand <- stack[[1]]
-    putRcmdr("commandStack", c(stack[-1], NA))
+    if (!keep) putRcmdr("commandStack", c(stack[-1], NA))
     lastCommand
 }
